#include "serverstatuswidget.h"
#include <QApplication>
#include <QtConcurrent/QtConcurrent>
#include <QPainter>
#include <QPaintEvent>

ServerStatusWidget::ServerStatusWidget(QWidget *parent) : QWidget(parent)
{
    setAttribute(Qt::WA_Hover);
    setMouseTracking(true);
    current_bg = QPixmap(":/flags/flag2.png");
    changeImageSrc(":/flags/flag2.png");
}

void ServerStatusWidget::changeImageSrc(const QString &src)
{
    if (!qApp)
        return;

    img_src = src;
    QPixmap *cur_bg = new QPixmap(current_bg);
    QPixmap *new_bg= new QPixmap(QPixmap(src));

    QtConcurrent::run([cur_bg, new_bg, src, this](){
        const int iterations_num = 100;
        const int iteration_sleep = 20;

        for (int i = 0; i < iterations_num && qApp; i++) {
            if (img_src != src)
                break;

            QPainter painter;
            painter.begin(cur_bg);
            painter.setOpacity(double(10 + i) / double(iterations_num));
            painter.setCompositionMode(QPainter::CompositionMode_Source);
            painter.drawPixmap(0,0, *new_bg);
            painter.end();
            bg_lock.lock();
            current_bg = *cur_bg;
            bg_lock.unlock();
            QMetaObject::invokeMethod(this, "repaint", Qt::QueuedConnection);
            QThread::msleep(iteration_sleep);
        }
        delete cur_bg;
        delete new_bg;
    });

}

void ServerStatusWidget::enterEvent(QEvent * event)
{
    qDebug() << Q_FUNC_INFO << this->objectName();
    changeImageSrc(":/flags/flag2_hover.png");
    QWidget::enterEvent(event);
}

void ServerStatusWidget::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QPainter painter;
    painter.begin(this);
    painter.drawPixmap(0,0, current_bg);
    painter.end();
}

void ServerStatusWidget::leaveEvent(QEvent * event)
{
    qDebug() << Q_FUNC_INFO << this->objectName();
    changeImageSrc(":/flags/flag2.png");
    QWidget::leaveEvent(event);
}

